package org.netkernel.http.session;

import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.Map;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class HTTPSessionMetadataEndpoint extends StandardAccessorImpl
{
	public static final String SESSION_STATUS_ID="active:httpSessionStatus";
	
	HTTPSessionOverlay mSessionContext;
	
	public HTTPSessionMetadataEndpoint(HTTPSessionOverlay aContext)
	{	mSessionContext=aContext;
		this.declareName("SessionMetadataEndpoint");
		this.declareDescription("Provides statistics for the HTTP sessions");
		this.declareSourceRepresentation(IBinaryStreamRepresentation.class);
	}
	
	public void onSource(INKFRequestContext helper) throws Exception
	{	long now=System.currentTimeMillis();
		switch(helper.getThisRequest().getVerb())
		{	case INKFRequestReadOnly.VERB_SOURCE:
			{	Map sessions=mSessionContext.getSessions();
				int size=sessions.size();
				StringBuilder sb=new StringBuilder();
				sb.append("<httpSessionStatus>");
				sb.append("<sessions>"+size+"</sessions>");
				int count=0;
				Iterator i=sessions.values().iterator();
				while(i.hasNext())
				{	try
					{	Session s=(Session)i.next();
						count+=s.getSessionContext(now).getItemCount();
					}
					catch(ConcurrentModificationException e)
					{	/*Ignore it - better to have some information from most sessions than none at all */
						size--;  //decrement session size to ensure average still correct
					}
				}
				sb.append("<totalObjects>"+count+"</totalObjects>");
				sb.append("<meanObjectsPerSession>"+count/size+"</meanObjectsPerSession>");
				sb.append("<minDuration>"+mSessionContext.getMinDuration()+"</minDuration>");
				sb.append("<maxDuration>"+mSessionContext.getMaxDuration()+"</maxDuration>");
				sb.append("<maxSessions>"+mSessionContext.getMaxSessions()+"</maxSessions>");
				sb.append("</httpSessionStatus>");
				INKFResponse resp=helper.createResponseFrom(new ByteArrayRepresentation(sb.toString().getBytes()));
				resp.setMimeType("text/xml");
				resp.setExpiry(INKFResponse.EXPIRY_MIN_CONSTANT_DEPENDENT, System.currentTimeMillis()+5000);  //Cache for 5 seconds
			}
			break;
			default:
				throw helper.createFormattedException("EX_UNSUPPORTED_VERB","MSG_SESSION_META_VERB", null, null, SESSION_STATUS_ID);
		}
	}
}
