package org.ten60.netkernel.servlet;

import com.ten60.netkernel.cache.se.representation2.ConcurrentCache;
import com.ten60.netkernel.cache.se.resolution.ResolutionCache;
import org.netkernel.container.IKernelListener;
import org.netkernel.container.ILogger;
import org.netkernel.container.impl.Kernel;
import org.netkernel.layer0.boot.IModuleFactory;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.logging.LogManager;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.impl.NKFTransportContextImpl;
import org.netkernel.layer0.tools.ExtraMimeTypes;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.Layer0Factory;
import org.netkernel.module.standard.StandardModuleFactory;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.INetKernelThrowable;
import org.netkernel.urii.ISpace;

import javax.servlet.ServletException;
import javax.servlet.UnavailableException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URI;
import java.net.URL;


public class NetKernelServlet extends HttpServlet
  {
  private static int SYSTEM_RUN_LEVEL_START = 1;
  private ModuleManager mModuleManager;
  private ConcurrentCache mRepresentationCache;
  private INKFRequestContext mContext;
           

  @Override
  public void init() throws ServletException
    {
    System.out.println("NetKernelServlet.init(): start");

    String filePathToModules = this.getServletContext().getRealPath("/modules");
    System.out.println("Real path to modules [" + filePathToModules + "]");
//    if (filePathToModules.contains(" "))
//    {	filePathToModules=filePathToModules.replaceAll("\\ ", "%20");
//    }
    System.out.println("Real path to modules after blank substitutions [" + filePathToModules + "]");

    File installDir = new File(filePathToModules);
    System.out.println(installDir);
    System.out.println(installDir.getAbsolutePath());
    System.out.println(installDir.toURI());

    // Create a new micro-kernel.
    Kernel kernel = new Kernel();

    // NetKernel adds support for missing MIME types to the JDK.
    ExtraMimeTypes.getInstance();

    // NetKernel requires a logger
    ILogger logger = new LogManager(null).getKernelLogger();

    ServletPropertyConfiguration config = new ServletPropertyConfiguration();

    // Set the default values
    config.setString("netkernel.scheduler.threadcount", "4" );
    config.setString("netkernel.scheduler.thread.timeout", "2000");
    config.setString("netkernel.scheduler.maxstackdepth", "20");
    config.setString("netkernel.debug","false");
    config.setString("netkernel.exception.javastack", "10");
    config.setString("netkernel.poll", "500" /*ms*/);
    config.setString("netkernel.statistics.period","2000");
    config.setString("netkernel.statistics.window","360");
    config.setString("netkernel.statistics.multiplier", "10");
    config.setString("se.concurrentcache.headroomMb","9" /*MB*/);
    config.setString("se.concurrentcache.cull%","33" /*%*/);
    config.setString("se.concurrentcache.maxsize", "6000");
    config.setString("se.resolutioncache.size", "200");
    config.setString("se.resolutioncache.interval", "100");
    config.setString("se.resolutioncache.touch","1000");
    config.setString("netkernel.instance.identifier", "NetKernel Embedded Servlet");
    config.setString("netkernel.instance.version.major","4");
    config.setString("netkernel.instance.version.minor","0");
    config.setString("netkernel.instance.product", "NetKernel");
    config.setString("netkernel.boot.time", Long.toString(System.currentTimeMillis()));

    // Read any user-set init-param information into the configuration
    setInitParameter(config, "netkernel.scheduler.threadcount");
    setInitParameter(config, "netkernel.scheduler.thread.timeout");
    setInitParameter(config, "netkernel.schedulre.maxstackdepth");
    setInitParameter(config, "netkernel.debug");
    setInitParameter(config, "netkernel.exception.javastack");
    setInitParameter(config, "netkernel.poll");
    setInitParameter(config, "netkernel.statistics.period");
    setInitParameter(config, "netkernel.statistics.window");
    setInitParameter(config, "netkernel.statistics.multiplier");
    setInitParameter(config, "se.concurrentcache.headroomMb");
    setInitParameter(config, "se.concurrentcache.cull%");
    setInitParameter(config, "se.concurrentcache.maxsize");
    setInitParameter(config, "se.resolutioncache.size");
    setInitParameter(config, "se.resolutioncaache.interval");
    setInitParameter(config, "se.resolutioncache.touch");
    setInitParameter(config, "netkernel.instance.identifier");

    kernel.setConfiguration(config);
    kernel.setLogger(logger);

    // Setup representation cache
    mRepresentationCache = new ConcurrentCache(kernel);
    kernel.setRepresentationCache(mRepresentationCache);
    kernel.addConfigurationListener(mRepresentationCache);

    // Setup resolution cache
    ResolutionCache resolutionCache = new ResolutionCache(kernel);
    kernel.setResolutionCache(resolutionCache);
    kernel.addConfigurationListener(resolutionCache);

    // Set up monitor for Visualizer to work
    IKernelListener monitor = Layer0Factory.createMonitor(kernel);
    kernel.setMonitor(monitor);

    // Instantiate the modules factories - only use StandardModule here.
    IModuleFactory[] factories = new IModuleFactory[]{new StandardModuleFactory()};

    // Create a Module Manager
    mModuleManager = new ModuleManager(kernel, factories);

    // Tell ModuleManager about the modules we want to use.
    InputStream is;
    try
      {
      URL modulesToLoad = this.getServletContext().getResource("/modules.conf");
      is = modulesToLoad.openStream();
      }
    catch (Exception e)
      {
      this.getServletContext().log("modules.conf file unavailable", e);
      throw new UnavailableException("NetKernel Servlet cannot read module configuration file.");
      }

    BufferedReader r = new BufferedReader(new InputStreamReader(is));
    String line;
    try
      {
      while ((line = r.readLine()) != null)
        {
        line = line.trim();
        if (!line.startsWith("#") && line.length() > 0)
          {
          try
            {
            File moduleFile = new File(installDir,line);
            URI moduleURI = moduleFile.toURI();
            System.out.println("Loading [" + moduleURI +  "]");
            mModuleManager.addModule(moduleURI, SYSTEM_RUN_LEVEL_START);
            }
          catch (Exception e)
            {
            this.getServletContext().log("Error with module reference [" + filePathToModules + "/" + line + "]", e);
            }
          }
        }
      }
    catch (Exception e)
      {
      this.getServletContext().log("Error occurred while processing module.conf file", e);
      throw new UnavailableException("NetKernel Servlet could not process module.conf file.");
      }

    // Set run level and make kernel live
    mModuleManager.setRunLevel(SYSTEM_RUN_LEVEL_START);

    // Create a transport with a reference to the Servlet Fulcrum rootspace
    IIdentifier spaceIdentifier = new SimpleIdentifierImpl("urn:org:netkernel:servlet:http");
    ISpace applicationRootSpace = kernel.getSpace(spaceIdentifier, null, null);
    mContext = new NKFTransportContextImpl(kernel, applicationRootSpace, this);

    System.out.println("NetKernelServlet.init(): end");
    }

  public void destroy()
    {
    System.out.println("NetKernelServlet.destroy(): start");
    mModuleManager.stop();
    mRepresentationCache.stop();
    System.out.println("NetKernelServlet.destroy(): end");
    }

  @Override
  protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    processHTTPRequest(request, response);
    }

  @Override
  protected void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    processHTTPRequest(request, response);
    }

  @Override
  protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    processHTTPRequest(request, response);
    }


  @Override
  protected void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    processHTTPRequest(request, response);
    }

  private void processHTTPRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    try
      {
      INKFRequest subRequest = mContext.createRequest(request.getRequestURL().toString());
      Object[] oArray = new Object[2];
      oArray[0] = request;
      oArray[1] = response;
      subRequest.addPrimaryArgument(oArray);
      mContext.issueRequestForResponse(subRequest);
      }
    catch (Throwable t)
      {
      response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
      //TODO: response.setDateHeader(HttpHeaders.EXPIRES, 0);
      response.setContentType("text/plain");
      //TODO: response.setHeader(HttpHeaders.SERVER, HTTPTransportEndpoint.getServerHeader());
      OutputStream os = response.getOutputStream();
      PrintWriter pw = new PrintWriter(os);
      if (t instanceof INetKernelThrowable)
        {
        pw.write(t.toString());
        }
      else
        {
        t.printStackTrace(pw);
        }
      pw.flush();
      }
    finally
      {
      response.flushBuffer();
      }
    }


  private void setInitParameter(ServletPropertyConfiguration configuration, String parameterName)
    {
    String parameterValue;
    parameterValue = getInitParameter(parameterName);
    System.out.println("Read init-param [" + parameterName + "] as [" + parameterValue + "]");
    if (parameterValue != null)
      {
      System.out.println("Setting parameter [" + parameterName + "] to [" + parameterValue + "]");
      configuration.setString(parameterName, parameterValue);
      }

    }
  }
